#include <iostream>
#include <cstdlib>
#include <SDL/SDL_mixer.h>

#include "Audio.h"

Mix_Music* Audio::_music[MUS_MAX];
Mix_Chunk* Audio::_sounds[SFX_MAX];
bool Audio::_disabled = false;

bool Audio::Init()
{
	if(Mix_Init(MIX_INIT_OGG) != MIX_INIT_OGG)
	{
		std::cout << "Error: Could not load SDL_Mixer (";
		std::cout << Mix_GetError() << ")" << std::endl;
		Audio::DisableSound();
		return false;
	}

	if(Mix_OpenAudio(44100,MIX_DEFAULT_FORMAT, 2,1024) == -1)
	{
		std::cout << "Error: Could not open audio device (";
		std::cout << Mix_GetError() << ")" << std::endl;
		Audio::DisableSound();
		return false;
	}

	if(!Audio::LoadSounds())
	{
		std::cout << "Error: Could not load sound effects (";
		std::cout << Mix_GetError() << ")" << std::endl;
		Audio::DisableSound();
		return false;
	}

	if(!Audio::LoadMusic())
	{
		std::cout << "Error: Could not load music (";
		std::cout << Mix_GetError() << ")" << std::endl;
		Audio::DisableSound();
		return false;
	}

	_disabled = false;
	return true;
}

bool Audio::LoadSounds()
{
	//Mix_LoadWAV(
	return true;
}

bool Audio::LoadMusic()
{
	/*Audio::_music[MUS_ENCOUNTER] = Mix_LoadMUS("res/mus/Encounter.mp3");
	Audio::_music[MUS_PARADOX] = Mix_LoadMUS("res/mus/Paradox.mp3");*/

	int i = 0;
	while (i < MUS_MAX)
	{
		if(!Audio::_music[i])
		{
			std::cout << "Error: Could not load music (";
			std::cout << Mix_GetError() << ")" << std::endl;
			Audio::FreeSounds();

			return false;
		}
		i++;
	}

	return true;
}

void Audio::PlayMusic(int id)
{
	if(!Audio::_disabled)
	{
		Mix_PlayMusic(Audio::_music[id], -1);
	}
}

void Audio::FreeSounds()
{
	std::cout << "Freeing sound" << std::endl;

	int i = 0;
	while(i < SFX_MAX)
	{
		if(Audio::_sounds[i])
		{
			Mix_FreeChunk(Audio::_sounds[i]);
		}
		i++;
	}

	std::cout << "Freeing music" << std::endl;

	int j = 0;
	while(j < MUS_MAX)
	{
		if(Audio::_music[j])
		{
			Mix_FreeMusic(Audio::_music[j]);
		}
		j++;
	}
}

void Audio::DisableSound()
{
	std::cout << "Disabling sound" << std::endl;
	Audio::_disabled = true;
}

void Audio::Close()
{
	Audio::FreeSounds();
	
	std::cout << "Closing SDL_mixer" << std::endl;
	Mix_Quit();
}
