#include <iostream>
#include <cmath>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Timer.h"
#include "Audio.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Bit.h"
#include "BitSocket.h"
#include "FrameStore.h"
#include "Frame.h"
#include "Letter.h"
#include "Text.h"
#include "Keyboard.h"
#include "GameState.h"

#include "Game.h"

BitSocket::BitSocket(Vector const& pos, Frame* parent, int flags):GameObject(pos)
{
	_t = Graphics::GetTexture(GFX_BIT);
	_width = _t->w;
	_height = _t->h;

	_sockRad = 16;
	_parentFrame = parent;
	_bit = NULL;

	_offset = pos - _parentFrame->GetPos();

	_special  = 1 & flags;
	_disabled = 2 & flags;
	_empty    = 4 & flags;

	if(_special)
	{
		_colour.g = 0;
		_colour.b = 0;
	}
	else if(_disabled)
	{
		_colour.r = 0;
	}
	if(!_empty)
	{
		CreateBit();
	}

	_colour.a = 0.3;
}

void BitSocket::CreateBit()
{
	if(_bit == NULL && _special == false && _disabled == false)
	{
		_bit = new Bit(_tr->pos);
		_bit->SetGroupID(_gID);
		_bit->SetRot(_tr->rot);
		Game::AddGameObject(_bit);
	}
}

void BitSocket::DeleteBit()
{
	if(_bit != NULL)
	{
		std::cout << "Deleting bit" << std::endl;
		Game::Remove(_bit);
		_bit = NULL;
		_empty = true;
		std::cout << "Bit deleted" << std::endl;
	}		
}

void BitSocket::FireBit()
{
	if(_bit != NULL)
	{
		Vector dir(0,-1);
		dir.Rotate2D(GetRot());

		Vector f = F_Fire(dir*-1);
		_bit->AddForce(f);
		_parentFrame->AddForce(f*-1);

		EmancipateBit();
	}
}

void BitSocket::Update()
{
	if(_bit != NULL)
	{
		AddForceTo(_bit);
		if(GetDistanceTo(_bit->GetPos()) > 16)
			EmancipateBit();
	}

	Move();
}

void BitSocket::EmancipateBit()
{
	if(_bit != NULL)
	{
		_bit->Die();
		_bit = NULL;
	}
}

Vector const& BitSocket::GetOffset()
{
	return _offset;
}
bool BitSocket::IsDisabled() const
{
	return _disabled;
}

bool BitSocket::IsSpecial() const
{
	return _special;
}

bool BitSocket::IsEmpty() const
{
	return _bit == NULL;
}

void BitSocket::AddForceTo(GameObject*const g)
{
	Vector p = g->GetCentre();
	Vector v = g->GetVel();
	Vector dir = GetDirectionTo(p);
	float d = GetDistanceTo(p);

	Vector f = F_Sock(d,v,dir);
	g->AddForce(f);
	_parentFrame->AddForce(f*-1);
}

void BitSocket::Move() { } // Movement handled by Frame::Move()

void BitSocket::SetGroupID(Uint32 id)
{
	GameObject::SetGroupID(id);
	if(_bit != NULL)
		_bit->SetGroupID(id);
}

void BitSocket::RotateAbout(Vector const& p, float a)
{
	Entity::RotateAbout(p,a);
	if(_bit != NULL)
	{
		_bit->Rotate(a);
	}
}

Vector BitSocket::F_Sock(float d, Vector const& v, Vector const& dir)
{
	static const float c = 0.0075;
	Vector f = Vector();
	if(d > 0.5 && d < _sockRad)
	{
		f = dir*d*c - v/10;
	}

	return f;
}

Vector BitSocket::F_Fire(Vector const& dir)
{
	return dir;
}
