#include <cstdlib>
#include <iostream>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <string>
#include <map>
#include <list>
#include <vector>
#include "Timer.h"
#include "Keyboard.h"
#include "Window.h"
#include "Texture.h"
#include "Transform.h"
#include "Colour.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Bit.h"

#include "GameState.h"
#include "Game.h"

#include "GravityWell.h"
#include "Blackhole.h"

Blackhole::Blackhole(Vector const& pos):GameObject(pos)
{
	_t = Graphics::GetTexture(GFX_BLACKHOLE);
	_width = _t->w;
	_height = _t->h;
	_mass = 1.66*pow(10,14);
	_radius = 150;
	_collides = true;

	_brightness = 0;

	_well = new GravityWell(this);
	_well->SetGroupID(_gID);
	Game::AddGameObject(_well);

	SpawnBits();
}

void Blackhole::IsInside(GameObject* g,Vector const& uRel, Vector const& dif)
{
	switch(g->GetType())
	{
	case ENT_PLAYER:
	case ENT_FRAME:
		break;
	default:
		Game::Remove(g);
		break;
	}
}

void Blackhole::SpawnBits()
{
	Uint32 nBits = rand() % 100 + 50;

	Bit *bit = NULL;

	for(int i = 0; i < nBits; ++i)
	{
		Uint32 distance = rand () % (int)round((_well->GetRadius() - _radius)) + _radius;
		Uint32 angle = rand() % 360;

		float fMag = F_Grav(1,_mass,distance,Vector(1,0)).Magnitude();
		float speed = sqrt(fMag*distance);

		Vector pos = Vector(0,distance);
		pos.Rotate2D(angle);
		Vector vel = pos.Versor() * speed;
		vel.Rotate2D(90);

		pos += GetCentre();

		bit = new Bit(pos);
		bit->SetVel(vel);
		bit->SetGroupID(_gID + 1);
		Game::AddGameObject(bit);
	}
}

Blackhole::~Blackhole()
{
	Game::Remove(_well);
}

float Blackhole::GetRadius() const { return _radius; }	
