#include <iostream>
#include <cmath>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Timer.h"
#include "Audio.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "Entity.h"
#include "GameObject.h"
#include "Enemy.h"
#include "Keyboard.h"
#include "GameState.h"

#include "Game.h"

const Uint32 Enemy::_deadTime = 1000;
const Vector Enemy::_releasePos = Vector(226,186);

Enemy::Enemy(Vector const& startPos,Colour const& c,Uint32 releaseTime = 0):GameObject(startPos),_startPos(startPos)
{
	_t = Graphics::GetTexture(GFX_UNICORN);
	_width = 10;
	_height = 10;
	_hitbox = new Rectangle(_width,_height);

	_collides = true;
	_type = ENT_UNICORN;

	_hasMoved = true;
	_releaseTime = releaseTime;
	_free = false;
	_dead = false;
	_colour = c;

	RandDir();
	_releaseTimer.Start();
}

void Enemy::RandDir()
{
	int dir = rand() % 4;

	std::cout << dir << std::endl;
	
	switch(dir)
	{
	case 0: _dir = DIR_LEFT; break;
	case 1: _dir = DIR_RIGHT; break;
	case 2: _dir = DIR_UP; break;
	case 3: _dir = DIR_DOWN; break;
	default: break;
	}
}

void Enemy::IsInside(GameObject* g)
{
	switch (g->GetType())
	{
		case ENT_WALL: MoveBack(); RandDir(); std::cout << "Och Nae" << std::endl; break;
		default: break;
	}
}

void Enemy::Die()
{
	_dead = true;
	_deadTimer.Start();
}

void Enemy::Release()
{
	_tr->pos = _releasePos;
	_free = true;
	_dead = false;
	_releaseTimer.Stop();
	_deadTimer.Stop();
}

void Enemy::Update()
{
	if(_dead)
	{
		_tr->pos = _startPos;
		if(_deadTimer.GetTicks() > _deadTime)
		{
			Release();
		}
	}
	else if(!_free)
	{
		_tr->pos = _startPos;
		if(_releaseTimer.GetTicks() > _releaseTime)
		{
			Release();
		}
	}
	else
	{
		Move();
	}
}

void Enemy::Move()
{
	/*if(_tr->pos.x == _lastPos.x && _tr->pos.y == _lastPos.y)
	{
		RandDir();
	}*/

	switch(_dir)
	{
	case DIR_LEFT:  _vel = Vector(-1, 0); break;
	case DIR_RIGHT: _vel = Vector( 1, 0); break;
	case DIR_UP:    _vel = Vector( 0,-1); break;
	case DIR_DOWN:  _vel = Vector( 0, 1); break;
	}

	_lastPos = _tr->pos;
	_tr->pos += _vel;
}
