#include <iostream>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Window.h"
#include "Graphics.h"

#include "Entity.h"

Entity::Entity(Vector const& pos)
{
	//Create transform and set position
	_tr = new Transform();
	_tr->pos = pos;
	_lastPos = pos;

	//Set default texture to NULL
	_t = NULL;
	_width = 1;
	_height = 1;
	_hitbox = NULL;

	//_dir = (0,-1);

	//Set default shading to opaque;
	_colour = Colour(1,1,1,1);
	_brightness = 1.0f;
	
	_type = ENT_NULL;
}

Entity::~Entity()
{
	delete _tr;
	if(_hitbox != NULL)
	{
		delete _hitbox;
	}
}

void Entity::MoveBack()
{
	_tr->pos = _lastPos;
}

void Entity::SetPos(Vector const& v)
{
	//Need to account for rotation
	_tr->pos = v;
}

Vector const& Entity::GetPos() const
{
	return _tr->pos;
}

Vector Entity::GetCentre() const
{
	return _tr->pos;
}

void Entity::SetColour(Colour const& c)
{
	_colour = c;
}

Colour const& Entity::GetColour() const
{
	return _colour;
}	

void Entity::SetBrightness(float a)
{
	_brightness = a;
}

float Entity::GetBrightness() const
{
	return _brightness;
}	

Texture const* Entity::GetTexture() const
{
	return _t;
}

Transform const* Entity::GetTransform() const
{
	return _tr;
}

bool Entity::HasMoved() const
{
	return _hasMoved;
}

Rectangle const* Entity::GetHitbox() const
{
	return _hitbox;
}

float Entity::GetWidth() const
{
	return _width;
}

float Entity::GetHeight() const
{
	return _height;
}
