#ifndef ENTITY_H
#define ENTITY_H

#define PI 3.14159

/* Notes
 * =====
 * Colour should be part of texture
 */
enum Type
{
	ENT_NULL,
	ENT_WALL,
	ENT_PLAYER,
	ENT_BIT,
	ENT_BIGBIT,
	ENT_UNICORN,
	ENT_MAX
};

class Entity
{
public:
	Entity(Vector const& pos);
	virtual ~Entity();

	//Update
	virtual void Update() = 0;
	void MoveBack();

	//Getters
	virtual float GetWidth() const;
	virtual float GetHeight() const;
	virtual float GetBrightness() const;
	virtual Vector GetCentre() const;
	virtual Vector const& GetPos() const;
	virtual Colour const& GetColour() const;
	virtual Transform const* GetTransform() const;
	virtual Texture const* GetTexture() const;
	virtual Rectangle const* GetHitbox() const;
	virtual bool HasMoved() const;

	//Setters
	virtual void SetPos(Vector const&);
	virtual void SetColour(Colour const&);
	virtual void SetBrightness(float);

	Type GetType() const { return _type; }
protected:
	//Position
	Transform *_tr;
	Vector _lastPos;
	Rectangle *_hitbox;
	float _width;
	float _height;

	Type _type;

	//Rendering
	Texture const* _t;
	Colour _colour;
	float _brightness;
	bool _hasMoved;
};
#endif
