#include <string>
#include <sstream>
#include <fstream>
#include <iostream>
#include <cmath>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Timer.h"
#include "Audio.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Bit.h"
#include "BitSocket.h"
#include "FrameStore.h"
#include "Frame.h"
#include "Keyboard.h"
#include "Letter.h"
#include "Text.h"
#include "FPSText.h"
#include "GameState.h"
#include "Game.h"

Frame::Frame(Vector const& pos, FrameType fType)
	:GameObject(pos)
{
	_sockW = 0;
	_sockH = 0;

	_type = ENT_FRAME;
	_fType = fType;

	_collides = true;

	FrameTemplate const* fTemplate = FrameStore::GetTemplate(fType);
	LoadFromTemplate(fTemplate);
}

Frame::~Frame()
{
	DestroySockets();
}

void Frame::LoadFromTemplate(FrameTemplate const*const fTemplate)
{
	//Basic settings
	_sockW = fTemplate->sockW;
	_sockH = fTemplate->sockH;
	_mass = fTemplate->mass;
	_t = fTemplate->t;

	//Bitsocket Loading
	BitSocket* s = NULL;
	Vector startPos = _tr->pos + fTemplate->socketOffset;
	Vector pos = startPos;
	int flags;

	for(int i = 0; i < _sockH; i++)
	{
		pos.x = startPos.x;
		for(int j = 0; j < _sockW; j++)
		{
			switch(fTemplate->socketTemplate[i][j])
			{
				case 'E': flags = 4; break; //Empty
				case 'S': flags = 1; break; //Special
				case 'O': flags = 0; break; //Occupied
				case 'D': flags = 2; break; //Disabled
				default:  flags = 2; break;
			}

			s = new BitSocket(pos, this, flags);
			_socketArray[i][j] = s;
			Game::AddGameObject(s);

			pos.x += 6;
		}
		pos.y += 6;
	}
}

void Frame::Update()
{
	HandleForces();
	Move();

	//Redo this?
	UpdateSockets();
	Vector dif = _tr->pos - _socketArray[0][0]->GetPos();
}

void Frame::Move()
{
	_tr->pos += _vel;
	for(int i = 0; i < _sockH; i++)
	{
		for(int j = 0; j < _sockW; j++)
		{
			Vector offset = _socketArray[i][j]->GetOffset();
			offset.Rotate2D(_tr->rot);
			_socketArray[i][j]->SetPos(_tr->pos + offset);
		}
	}
}

void Frame::SetGroupID(Uint32 id)
{
	GameObject::SetGroupID(id);

	for(int i = 0; i < _sockH; ++i)
	{
		for(int j = 0; j < _sockW; ++j)
		{
			_socketArray[i][j]->SetGroupID(id);
		}
	}
}

bool Frame::AddFuelBit()
{
	for(int i = 0; i < _sockH; i++)
	{
		for(int j = 0; j < _sockW; j++)
		{
			int fuelSlot = _socketArray[i][j]->IsEmpty();
			    fuelSlot &= !_socketArray[i][j]->IsDisabled();
			    fuelSlot &= !_socketArray[i][j]->IsSpecial();
			if(fuelSlot) 
			{
				_socketArray[i][j]->CreateBit();
				return true;
			}
		}
	}

	return false;
}

bool Frame::RemoveFuelBit()
{
	for(int i = _sockH - 1; i >= 0; i--)
	{
		for(int j = _sockW - 1; j >= 0; j--)
		{
			if(!_socketArray[i][j]->IsEmpty())
			{
				_socketArray[i][j]->DeleteBit();
				return true;
			}
		}
	}

	return false;
}

void Frame::FireFuel()
{
	for(int i = _sockH - 1; i >= 0; i--)
	{
		for(int j = _sockW - 1; j >= 0; j--)
		{
			if(!_socketArray[i][j]->IsEmpty())
			{
				if(!_socketArray[i][j]->IsSpecial())
				{
					_socketArray[i][j]->FireBit();
					return;
				}
			}
		}
	}
}

void Frame::RotateAbout(Vector const& point, float a)
{
	Entity::RotateAbout(point,a);

	for(int i = 0; i < _sockH; i++)
	{
		for(int j = 0; j < _sockW; j++)
		{
			_socketArray[i][j]->RotateAbout(point,a);
		}
	}
}

void Frame::Stop()
{
	//_vel = 0;
}

void Frame::UpdateSockets()
{
	/*for(int i = 0; i < _sockH; i++)
	{
		for(int j = 0; j < _sockW; j++)
		{
			//Vector pos = _socketArray[i][j]->GetPos();
			//pos += _vel;
			//_socketArray[i][j]->SetPos(pos);
			_socketArray[i][j]->SetVel(_vel);
		}
	}*/
}

void Frame::DestroySockets()
{
	for(int i = 0; i < _sockH; i++)
	{
		for(int j = 0; j < _sockW; j++)
		{
			Game::Remove(_socketArray[i][j]);
		}
	}
}

void Frame::IsInside(GameObject* g,Vector const& uRel, Vector const& dif)
{
	if(g->GetType() == ENT_BIT && g->GetVel().Magnitude() < 2)
	{
		AddFuelBit();/*
		DestroyBit();*/
	}
}
