#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include <sstream>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include "Vector.h"
#include "Texture.h"
#include "Graphics.h"
#include "FrameStore.h"

bool FrameStore::_initialized = false;
std::map<FrameType,FrameTemplate const*> FrameStore::_templates;

bool FrameStore::Initialize()
{
	bool error = false;

	if(!_initialized)
	{
		for(int i = 0; i < FRAME_MAX; i++)
		{
			if(!LoadTemplate(i))
			{
				error = true;
				Free();
			}
		}

		_initialized = !error;
	}

	return _initialized;
}

void FrameStore::Free()
{
	//while(!_templates.empty())
	{
		//May not be quite right
		//delete (FrameTemplate*) *(_templates.end);
		//_templates.pop_back();
	}
}

bool FrameStore::LoadTemplate(int i)
{
	bool error = false;
	FrameTemplate* fTemplate = NULL;
	FrameType frameType;
	std::string filename;
	Texture const* t = NULL;

	switch(i)
	{
		case 0:
			frameType = FRAME_ACE;
			t = Graphics::GetTexture(GFX_ACE);
			filename = "Ace.txt";
			break;
		case 1:
			frameType = FRAME_APERTURE;
			t = Graphics::GetTexture(GFX_APERTURE);
			filename = "Aperture.txt";
			break;
		case 2: 
			frameType = FRAME_CAPSULE;
			t = Graphics::GetTexture(GFX_CAPSULE);
			filename = "Capsule.txt";
			break;
		case 3: 
			frameType = FRAME_DISCUS;
			t = Graphics::GetTexture(GFX_DISCUS);
			filename = "Discus.txt";
			break;
		case 4: 
			frameType = FRAME_EXCALIBUR;
			t = Graphics::GetTexture(GFX_EXCALIBUR);
			filename = "Excalibur.txt";
			break;
		case 5: 
			frameType = FRAME_KERNEL;
			t = Graphics::GetTexture(GFX_KERNEL);
			filename = "Kernel.txt";
			break;
		case 6: 
			frameType = FRAME_MONARCH;
			t = Graphics::GetTexture(GFX_MONARCH);
			filename = "Monarch.txt";
			break;
		case 7:
			frameType = FRAME_NEEDLE;
			t = Graphics::GetTexture(GFX_NEEDLE);
			filename = "Needle.txt";
			break;
		case 8:
			frameType = FRAME_ROGUE;
			t = Graphics::GetTexture(GFX_ROGUE);
			filename = "Rogue.txt";
			break;
		case 9:
			frameType = FRAME_SEPTER;
			t = Graphics::GetTexture(GFX_SEPTER);
			filename = "Septer.txt"; 
			break;
		case 10:
			frameType = FRAME_STING;
			t = Graphics::GetTexture(GFX_STING);
			filename = "Sting.txt";
			break;
		case 11:
			frameType = FRAME_THERMIDOR;
			t = Graphics::GetTexture(GFX_THERMIDOR);
			filename = "Thermidor.txt";
			break;
		case 12:
			frameType = FRAME_TRIDENT;
			t = Graphics::GetTexture(GFX_TRIDENT);
			filename = "Trident.txt";
			break;
		case 13:
			frameType = FRAME_WRAITH;
			t = Graphics::GetTexture(GFX_WRAITH);
			filename = "Wraith.txt";
			break;
		default:
			filename = "";
			break;
	}

	if(filename == "")
	{
		error = true;
		std::cout << "Error: Could not load frame (";
		std::cout << "Frame does not exist)" << std::endl;
	}
	else
	{
		fTemplate = LoadFromFile(filename.c_str());
	}

	if(!fTemplate)
	{
		error = true;
	}
	else
	{
		fTemplate->t = t;
		_templates[frameType] = fTemplate;
	}

	return !error;
}

FrameTemplate* FrameStore::LoadFromFile(char const* filename)
{
	FrameTemplate *fTemplate = NULL;
	std::stringstream s;

	s << "res/txt/" << filename;
	std::fstream file;
	file.open(s.str().c_str(),std::ifstream::in);

	if(file.fail())
	{
		std::cout << "Error: Could not load frame (";
		std::cout << "(File " << filename << " not found)" << std::endl;
		file.close();

		return NULL;
	}

	fTemplate = new FrameTemplate();

	file >> fTemplate->sockH; file.ignore(1,'X');
	file >> fTemplate->sockW; file.ignore(1,'\n');
	file >> fTemplate->socketOffset.x; file.ignore(1,',');
	file >> fTemplate->socketOffset.y; file.ignore(1,'\n');

	char socket;
	char next = file.peek();
	bool nl;
	bool error = false;

	//Change this to only fill from relevant
	//limits
	for (int i = 0; i < fTemplate->sockH; i++)
	{
		for (int j = 0; j < fTemplate->sockW; j++)
		{
			if(!nl && !error)
			{
				//Change to check EOF first
				socket = file.get();

				/*if(c == '\n')
				{
					//Replace with break
					nl = true;
				}
				else if(file.eof())
				{
					error = true;
				}
				else*/
				{
					if(ValidBS(socket))
					{
						fTemplate->socketTemplate[i][j] = socket;
					}
					else
					{
						std::cout << "Error: Could not load frame (";
						std::cout << "Invalid frame blueprint)" << std::endl;
						fTemplate = NULL;
						error = true;
					}
				}
			}
			else
			{
				break;
			}
		}
		next = file.peek();

		while(next == '\n' || next == '\r')
		{
			file.get();
			next = file.peek();
		}

		if(next == EOF)
		{
			error = true;
		}
		if(error)
		{
			break;
		}
	}

	file.close();

	return fTemplate;
}

bool FrameStore::ValidBS(char c)
{
	bool valid;

	switch(c)
	{
	case 'E':
	case 'S':
	case 'O':
	case 'D':
		valid = true;
		break;
	default:
		valid = false;
		break;
	}

	return valid;
}

FrameTemplate const* FrameStore::GetTemplate(FrameType fType)
{
	return _templates[fType];
}
