#include <iostream>
#include <cmath>
#include <ctime>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Timer.h"
#include "Audio.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Bit.h"
#include "BitSocket.h"
#include "FrameStore.h"
#include "Frame.h"
#include "Keyboard.h"
#include "Shell.h"
#include "Player.h"
#include "Letter.h"
#include "Text.h"
#include "FPSText.h"
#include "GameState.h"
#include "MainState.h"

#include "Game.h"

GameState* Game::_state = NULL;
Keyboard* Game::_keyboard = NULL;
Window* Game::_window = NULL;
Render* Game::_render = NULL;
Timer Game::_gameTimer;
bool Game::_run = true;
bool Game::_restart = false;

Game::Game() { }

float Game::_fps = 0;

//Create a window, a rendering context,
//load the graphics, create an audio 
//context and load sound. If all okay,
//create keyboard and start the game.
//If not okay, quit.
void Game::Initialize()
{
	srand(time(NULL));

	_window = new Window();
	if(!_window->Init())
	{
		_run = false;
	}

	_render = new Render();
	if(!_render->Init())
	{
		_run = false;
	}

	if(!Graphics::LoadGraphics())
	{
		_run = false;
	}

	_keyboard = new Keyboard();
	
	//Move this to a SetGameState function
	_state = new MainState(_keyboard, _render,_window);
	if(!_state->Initialize())
	{
		_run = false;
	}

	if(_run)
	{
		std::cout << "Game initialization successful" << std::endl;
	}
	else
	{
		std::cout << "Game initialization failed" << std::endl;
	}
}

//Delete the current GameState, close audio,
//delete keyboard, rendering context and close
//the window.
void Game::Close()
{
	std::cout << "Shutting down game" << std::endl;

	if(_state != NULL)
	{
		delete(_state);
	}

	Audio::Close();
	Graphics::UnloadGraphics();

	delete(_keyboard);
	delete(_render);
	delete(_window);
}

void Game::Run()
{
	SDL_Event event;

	while (_run)
	{
		_gameTimer.Start();
		_state->Update();

		while(SDL_PollEvent(&event))
		{
			switch(event.type)
			{
				case SDL_QUIT:
					_run = false;
					break;
				case SDL_KEYDOWN:
					_keyboard->KeyPress(event.key.keysym.sym);
					break;
				case SDL_KEYUP:
					_keyboard->KeyRelease(event.key.keysym.sym);
					break;
				default:
					break;
			}
		}

		if(_keyboard->IsPressed(SDLK_ESCAPE))
		{
			_run = false;
		}

		while(_gameTimer.GetTicks() < 16)
		{
		}

		_fps = 1000.0f/(float)(_gameTimer.GetTicks());
		if(_restart)
		{
			DoRestart();
		}
	}
}

float Game::GetFPS()
{
	return _fps;
}

//Provides a global accessor for current
//state's entity lists.
//TODO Make it better
void Game::AddObject(Entity* ent)
{
	_state->AddObject(ent);
}

void Game::AddGameObject(GameObject* ent)
{
	_state->AddGameObject(ent);
}

void Game::Remove(Entity* ent)
{
	_state->RemoveObjectLater(ent);
}

void Game::Restart()
{
	_restart = true;
}
void Game::DoRestart()
{
	delete _state;
	_state = new MainState(_keyboard, _render,_window);
	_state->Initialize();

	_restart = false;
}

