#include <iostream>
#include <cmath>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Line.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "GameObject.h"
#include "Render.h"
#include "Keyboard.h"

#include "GameState.h"

GameState::GameState(Keyboard const* k, Render *const r, Window *const w):_keyboard(k),_lastState(new Keyboard(*k)),_render(r),_window(w)
{
}

GameState::~GameState()
{
	std::cout << "Deleting Objects" << std::endl;
	DestroyObjects();
}

void GameState::Update()
{
	HandleEvents();
	UpdateObjects();
	DrawObjects();
	delete(_lastState);
	_lastState = new Keyboard(*_keyboard);
}

void GameState::AddObject(Entity *const obj)
{
	_objects.push_back(obj);
}

void GameState::AddGameObject(GameObject *const obj)
{
	_gameObjects.push_back(obj);
}

void GameState::RemoveObjectLater(Entity *const obj)
{
	bool exists = false;

	if(!_toRemove.empty())
	{
		for (int i = 0; i < _toRemove.size(); ++i)
		{
			if(_toRemove[i] == obj)
			{
				exists = true;
			}
		}
	}
	if(!exists)
	{
		_toRemove.push_back(obj);
	}
}

void GameState::RemoveDeleted()
{
	while(!_toRemove.empty())
	{
		Entity* e = _toRemove.back();
		std::list<Entity*>::iterator iter;
		for(iter = _objects.begin(); iter != _objects.end(); ++iter)
		{
			if(*iter == e)
			{
				_objects.remove(*iter);
				break;
			}
		}
		std::list<GameObject*>::iterator iter2;
		for(iter2 = _gameObjects.begin(); iter2 != _gameObjects.end(); ++iter2)
		{
			if(*iter2 == e)
			{
				_gameObjects.remove(*iter2);
				break;
			}
		}
		_toRemove.pop_back();
		delete e;
	}
}

void GameState::UpdateObjects()
{
	/*std::list<Entity*>::iterator iter;
	for( iter = _objects.begin(); iter != _objects.end(); iter++)
	{
		(*iter)->Update();
	}
	CheckCollisions();*/
}

void GameState::DrawObjects()
{
	/*std::list<Entity*>::iterator iter;

	for( iter = _objects.begin(); iter != _objects.end(); iter++)
	{
		_render->Draw(*iter);
	}

	_render->Refresh();*/
}

void GameState::DestroyObjects()
{
	while(!_objects.empty())
	{
		Entity* e = _objects.back();
		_objects.pop_back();
		delete e;
	}
	while(!_gameObjects.empty())
	{
		Entity* e = _gameObjects.back();
		_gameObjects.pop_back();
		delete e;
	}
}

/*bool GameState::TestCollision(GameObject const*const A,GameObject const*const B)
{
	if(A->GetGroupID() == B->GetGroupID()) return false;
	if(!A->Collides() || !B->Collides()) return false;
	if(A->GetHitbox == NULL || B->GetHitbox == NULL);

	Vector const& aP = A->GetPos();
	Vector const& bP = B->GetPos();

	float aW = A->GetWidth() + aP.x;
	float bW = B->GetWidth() + bP.x;
	float aH = A->GetHeight() + aP.y;
	float bH = B->GetHeight() + bP.y;

	if(aW < bP.x || aP.x > bW) return false;
	if(aH < bP.y || aP.y > bH) return false;

	return CollisionDetector::Test(*(A->GetHitbox),A->GetTransform(),*(B->GetHitbox()),B->GetTransform());
}*/
