#include <iostream>
#include <cmath>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <sstream>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Line.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Timer.h"
#include "Audio.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Keyboard.h"
#include "Wall.h"
#include "Map.h"
#include "Bit.h"
#include "Player.h"
#include "Letter.h"
#include "Text.h"
#include "GameState.h"
#include "Enemy.h"

#include "CollisionDetector.h"

#include "MainState.h"

MainState::MainState(Keyboard const *k, Render *r,Window *w):GameState(k,r,w)
{
	_initialized = false;
	_paused = false;
	_player = NULL;
	_lives = 3;
	_score = 0;
}

MainState::~MainState()
{
	DestroyObjects();
}

bool MainState::Initialize()
{
	_player = new Player(Vector(246,376),_keyboard);
	
	Map* map = new Map(Vector());
	AddGameObject(map);
	map->LoadMap();
	AddGameObject(_player);
	AddGameObject(new Enemy(Vector(226,186),Colour(1,0,0),0));
	AddGameObject(new Enemy(Vector(224,233),Colour(1,0.7,1),5000));
	AddGameObject(new Enemy(Vector(250,233),Colour(0,1,1),10000));
	AddGameObject(new Enemy(Vector(198,233),Colour(1,0.7,0.3),15000));

	return true;
}

void MainState::Update()
{
	HandleEvents();

	std::list<GameObject*>::iterator iter1;
	std::list<Entity*>::iterator iter2;

	for( iter1 = _gameObjects.begin(); iter1 != _gameObjects.end(); ++iter1)
	{
		if(!_paused)
			(*iter1)->Update();
	}

	CheckCollisions();
	RemoveDeleted();

	for( iter1 = _gameObjects.begin(); iter1 != _gameObjects.end(); ++iter1)
	{
		_render->Draw(*iter1);
	}

	_render->Refresh();

	delete(_lastState);
	_lastState = new Keyboard(*_keyboard);
}

void MainState::HandleEvents()
{
	if(_keyboard->IsPressed('p'))
	{
		if(_lastState->IsReleased('p'))
		{
			Pause(!_paused);
		}
	}
}

void MainState::Pause(bool p)
{
	if(p == true)
	{
		std::cout << "Pausing" << std::endl;
	}
	else
	{
		std::cout << "Not pausing" << std::endl;
	}

	_paused = p;
}

void MainState::Resume()
{
	_paused = false;
}

void MainState::CheckCollisions()
{
	std::list<GameObject*>::iterator iter2;
	std::list<GameObject*>::iterator end = _gameObjects.end();

	for(std::list<GameObject*>::iterator iter1 = _gameObjects.begin(); iter1 != end; ++iter1)
	{
		if(!(*iter1)->Collides()) continue;
		if(!(*iter1)->HasMoved()) continue;

		Rectangle const* boxA = (*iter1)->GetHitbox();
		if(boxA == NULL) continue;

		Transform const* trA = (*iter1)->GetTransform();
		Uint32 gID = (*iter1)->GetGroupID();

		for(iter2 = _gameObjects.begin(); iter2 != end; ++iter2)
		{
			if(gID == (*iter2)->GetGroupID()) continue;
			if(!(*iter2)->Collides()) continue;

			Rectangle const* boxB = (*iter2)->GetHitbox();
			if(boxB == NULL) continue;

			Transform const* trB = (*iter2)->GetTransform();

			if(CollisionDetector::Test(*boxA,*trA,*boxB,*trB))
			{
				(*iter1)->IsInside((*iter2));
				(*iter2)->IsInside((*iter1));
			}
		}
	}
}
