#include <iostream>
#include <fstream>
#include <map>
#include <vector>
#include <list>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Keyboard.h"
#include "Timer.h"
#include "GameState.h"
#include "Game.h"
#include "Wall.h"
#include "Bit.h"
#include "BigBit.h"

#include "Map.h"

Map::Map(Vector const& pos):GameObject(pos)
{
	_t = Graphics::GetTexture(GFX_MAP);
	_width = _t->w;
	_height = _t->h;
	_tr->pos.x += _width / 2; 
	_tr->pos.y += _height / 2; 
}

Map::~Map() {}

void Map::LoadMap()
{
	std::fstream file;
	file.open("res/txt/map",std::ifstream::in);

	if(file.fail())
	{
		std::cout << "Error: Could not load map" << std::endl;
		file.close();

		return;
	}

	char socket;
	char next = file.peek();
	bool nl;
	bool error = false;
	Vector entPos = _tr->pos + 8;
	entPos.x -= _width / 2;
	entPos.y -= _height /2;

	for (int i = 0; i < 31; i++)
	{
		nl = false;
		for (int j = 0; j < 28; j++)
		{
			if(!nl && !error)
			{
				//Change to check EOF first
				socket = file.get();

				if(socket == '\n' || socket == '\r')
				{
					//Replace with break
					nl = true;
					std::cout << "New Line" << std::endl;
				}/*
				else if(file.eof())
				{
					error = true;
				}
				else*/
				if(!nl)
				{
					switch(socket)
					{
					case 'X':
						Game::AddGameObject(new Wall(Vector(entPos.x + j * 16, entPos.y + i * 16)));
						break;
					case 'D':
						Game::AddGameObject(new Bit(Vector(entPos.x + j * 16, entPos.y + i * 16)));
						break;
					case 'B':
						Game::AddGameObject(new BigBit(Vector(entPos.x + j * 16, entPos.y + i*16)));
						break;
					case 'S': break;
					default:
						std::cout << "Error: unexpected character in map file: ";
						std::cout << socket << std::endl;
						error  = true;
						break;
					}
				}
			}
			else
			{
				break;
			}
		}
		next = file.peek();

		while(next == '\n' || next == '\r')
		{
			file.get();
			next = file.peek();
		}

		if(next == EOF)
		{
			std::cout << "EOF reached" << std::endl;
			error = true;
		}
		if(error)
		{
			std::cout << "Error!" << std::endl;
			break;
		}
	}

	file.close();
}
