#include <iostream>
#include <map>
#include <vector>
#include <list>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Keyboard.h"
#include "Timer.h"
#include "Enemy.h"
#include "GameState.h"
#include "Game.h"

#include "Player.h"

Player* Player::_player = NULL;

Player::Player(Vector const& pos,Keyboard const*const keyboard):GameObject(pos), _keyboard(keyboard)
{
	_lastState = new Keyboard();

	_speed = 1;
	_big = false;
	_bigTime = 10000;
	_t = Graphics::GetTexture(GFX_GWB);
	_width = 10;
	_height = 10;
	_hitbox = new Rectangle(_width,_height);

	_type = ENT_PLAYER;
	_collides = true;

	if(_player == NULL)
	{
		_player = this;
	}
}

Player::~Player()
{
	if(_player == this)
	{
		_player = NULL;
	}

	delete _lastState;
}

Player const* Player::GetPlayer()
{
	return _player;
}

void Player::Update()
{
	if(_keyboard->IsPressed('w'))
	{
		_vel = Vector(0,-_speed);
	}
	else if(_keyboard->IsPressed('s'))
	{
		_vel = Vector(0,_speed);
	}
	else if(_keyboard->IsPressed('a'))
	{
		_vel = Vector(-_speed,0);
	}
	else if(_keyboard->IsPressed('d'))
	{
		_vel = Vector(_speed,0);
	}
	if(_vel.x == 0 && _vel.y == 0)
	{
		_hasMoved = false;
	}
	else
	{
		_hasMoved = true;
	}

	if(_big)
	{
		float r,g,b;

		r = ((float)(rand()%100))/100;
		g = ((float)(rand()%100))/100;
		b = ((float)(rand()%100))/100;

		_colour = Colour(r,g,b);

		if(_bigTimer.GetTicks() > _bigTime)
		{
			Unbig();
		}
	}

	Move();

	delete _lastState;
	_lastState = new Keyboard(*_keyboard);
}

void Player::Move()
{
	_lastPos = _tr->pos;

	_tr->pos += _vel;
}

void Player::Die()
{
	Game::Restart();
}

void Player::IsInside(GameObject* g)
{
	switch(g->GetType())
	{
	case ENT_UNICORN:
		if(_big)
		{
			g->Die();
		}
		else
		{
			Die();
		}
		break;
	case ENT_BIT:
		//_state->IncreaseScore();
		break;
	case ENT_BIGBIT:
		Embiggen();
		break;
	case ENT_WALL:
		MoveBack();
		break;
	default: break;
	}
}

void Player::Embiggen()
{
	_big = true;
	_bigTimer.Start();
}

void Player::Unbig()
{
	_big = false;
	_bigTimer.Stop();
	_colour = Colour(1,1,1);
}
