#include <iostream>
#include <SDL/SDL.h>
#include <GL/gl.h>
#include "Transform.h"
#include "Rectangle.h"
#include "Texture.h"
#include "Colour.h"
#include "Entity.h"

#include "Render.h"

Render::Render() {}
Render::~Render() { }

bool Render::Init()
{
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glMatrixMode(GL_PROJECTION);
	glOrtho(0,448,496,0,-1,1);

	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_DEPTH);
	glClearColor(0.0f,0.0f,0.0f,0.0f);

	glMatrixMode(GL_MODELVIEW);

	return true;
}

void Render::SetViewport(int w, int h)
{
	std::cout << "Changing viewport dimensions: ";
	std::cout << w << ", " << h << std::endl;
	glViewport(0,0,w,h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,w,h,0,-1,1);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_DEPTH);
	glClearColor(0.0f,0.0f,0.0f,0.0f);
}


void Render::Draw(Entity const *const ent, Transform const*const camera)
{
	Texture const* t = ent->GetTexture();
	Transform const* tr = ent->GetTransform();

	if(t == NULL || tr == NULL)
	{
		return;
	}

	//Retrieve properties
	Transform cam;
	Colour colour = ent->GetColour();
	float brightness = ent->GetBrightness();
	colour.r *= brightness;
	colour.g *= brightness;
	colour.b *= brightness;

	if(camera != NULL)
	{
		cam = *camera;
	}

	//if(t->alpha)

	glBindTexture(GL_TEXTURE_2D, t->id);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//glRotatef(-cam.rot,0,0,1);
	glTranslatef(-cam.pos.x,-cam.pos.y,0.0f);

	glTranslatef(tr->pos.x, tr->pos.y,0.0f);

	glScalef(tr->scale.x,tr->scale.y,1.0f);

	glRotatef(tr->rot,0,0,1);

	glTranslatef(((float)-t->w)/2.0f,((float)-t->h)/2.0f,0.0f);

	glScalef(t->w,t->h,1);


	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();

	glScalef(t->su,t->sv,1);

	glBegin(GL_QUADS);
	glColor4f(colour.r,colour.g,colour.b,colour.a);
	glTexCoord2f(0.0f,0.0f); glVertex2f(0.0f,0.0f);
	glTexCoord2f(1.0f,0.0f); glVertex2f(1.0f,0.0f); 
	glTexCoord2f(1.0f,1.0f); glVertex2f(1.0f,1.0f); 
	glTexCoord2f(0.0f,1.0f); glVertex2f(0.0f,1.0f); 
	glEnd();

	glMatrixMode(GL_MODELVIEW);
}

void Render::Refresh()
{
	SDL_GL_SwapBuffers();

	glClear(GL_COLOR_BUFFER_BIT);
}
