#include <cstdlib>
#include <iostream>
#include <cmath>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Timer.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Keyboard.h"
#include "GameState.h"
#include "Game.h"

#include "Star.h"

#include "StarMap.h"

StarMap::StarMap(Window const* w,Transform const* cam)
:GameObject(Vector()),_window(w),_camera(cam),_STAR_PROB(0.000813802f)
{
	

	_xProb = 0;
	_yProb = 0;
	_starSize = 7;

	_oldCam = new Transform();
	*_oldCam = *cam;

	/*const int sWidth = _window->GetWidth();
	const int sHeight = _window->GetHeight();

	Star* s = NULL;

	float brightness;
	float x = 0, y = 0;
	float d;

	for(int i = 0; i < 250; i++)
	{
		brightness = rand() % 100;
		brightness /= 100;

		x = (rand() % sWidth) + _camera->pos.x;
		y = (rand() % sHeight) + _camera->pos.y;

		d = rand() % 25 + 75;
		d /= 100;

		s = new Star(Vector(x,y),brightness,d,_camera); 
		_stars.push_back(s);
		Game::AddGameObject(s);
	}*/

	RedrawStars();
}

StarMap::~StarMap()
{
	delete _oldCam;
}

void StarMap::Update()
{
	//Add new stars
	Vector dif = _camera->pos - _oldCam->pos;

	float xBox;
	float yBox;
	float xyBox;

	xBox = dif.x * _window->GetHeight();
	yBox = dif.y * _window->GetWidth();
	xyBox = dif.x * dif.y;

	float xPixels = fabs(xBox - xyBox);
	float yPixels = fabs(yBox);

	_xProb += xPixels * _STAR_PROB;
	_yProb += yPixels * _STAR_PROB;

	Vector start;
	Vector end;

	if(_xProb >= 1)
	{
		int xStars = floor(_xProb);
		_xProb -= xStars;

		if(dif.x > 0)
		{
			start.x = _window->GetWidth() - dif.x + _starSize;
			end.x = _window->GetWidth() + _starSize;
		}
		else if(dif.x < 0)
		{
			start.x = 0 - _starSize;
			end.x = 0 - dif.x - _starSize;
		}
		else
		{
			start.x = 0;
			end.x = 0;
		}

		GenStars(Vector(start.x,0),Vector(end.x,_window->GetHeight() - dif.y),xStars);
	}
	
	if(_yProb >= 1)
	{
		int yStars = floor(_yProb);
		_yProb -= yStars;

		if(dif.y > 0)
		{
			start.y = _window->GetHeight() - dif.y + _starSize;
			end.y = _window->GetHeight() + _starSize;
		}
		else if(dif.y < 0)
		{
			start.y = 0 - _starSize;
			end.y = 0 - dif.y - _starSize;
		}
		else
		{
			start.y = 0;
			end.y = 0;
		}

		GenStars(Vector(0,start.y),Vector(_window->GetWidth(),end.y),yStars);
	}
	
	//Delete Stars
	std::vector<Star*> toRemove;

	float leftEdge = 0 - _starSize + _camera->pos.x;
	float topEdge = 0 - _starSize + _camera->pos.y;
	float rightEdge = _window->GetWidth() + _starSize + _camera->pos.x;
	float bottomEdge = _window->GetHeight() + _starSize + _camera->pos.y;

	std::list<Star*>::iterator iter;
	for(iter = _stars.begin(); iter != _stars.end(); ++iter)
	{
		Vector const& sPos = (*iter)->GetPos();

		if(sPos.x < leftEdge || sPos.y < topEdge
			|| sPos.x > rightEdge || sPos.y > bottomEdge)
		{
			toRemove.push_back(*iter);
		}
	}
	while(!toRemove.empty())
	{
		_stars.remove(toRemove.back());
		Game::Remove(toRemove.back());
		toRemove.pop_back();
	}

	//Store cam pos for parallax calculations
	*_oldCam = *_camera;
}

void StarMap::RedrawStars()
{
	while(!_stars.empty())
	{
		Game::Remove(_stars.back());
		_stars.pop_back();
	}

	const int w = _window->GetWidth();
	const int h = _window->GetHeight();

	int stars = _STAR_PROB*w*h*10;

	GenStars(Vector(0,0),Vector(w,h),stars);
}

void StarMap::GenStars(Vector const& start, Vector const& end, int nStars)
{
	int rangeX = ceil(end.x - start.x);
	int rangeY = ceil(end.y - start.y);

	if(rangeX == 0)
		return;
	if(rangeY == 0)
		return;

	Star* s = NULL;

	float brightness;
	float x = 0, y = 0;
	float d;

	float cull;
	float chance;

	for(int i = 0; i < nStars; ++i)
	{
		brightness = rand() % 100;
		brightness /= 100;

		x = (rand() % rangeX) + start.x + _camera->pos.x;
		y = (rand() % rangeY) + start.y + _camera->pos.y;

		d = rand() % 25 + 75;
		d /= 100;

		cull = 1.0f - d;
		chance = rand() % 100;
		chance /= 100;

		if(cull > chance)
		{
			s = new Star(Vector(x,y),brightness,d,_camera); 
			_stars.push_back(s);
			Game::AddGameObject(s);
		}
	}
}
