#include <iostream>
#include <cmath>
#include <list>
#include <vector>
#include <deque>
#include <map>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "Transform.h"
#include "Texture.h"
#include "Vector.h"
#include "Rectangle.h"
#include "Colour.h"
#include "Timer.h"
#include "Audio.h"
#include "Window.h"
#include "Graphics.h"
#include "Entity.h"
#include "Render.h"
#include "GameObject.h"
#include "Keyboard.h"
#include "Letter.h"
#include "Text.h"
#include "GameState.h"
#include "Game.h"

Text::Text(Vector pos, std::string text, Font font, int size):Entity(pos),_text(text),_font(font),_size(size)
{
	CreateLetters();
}

Text::~Text()
{
	DestroyLetters();
}

void Text::CreateLetters()
{
	Vector tPos = _tr->pos;

	for (int i = 0; i < _text.size(); i++)
	{
		Letter *l = new Letter(tPos,_text[i],_font,_size,this);

		Texture const* lT = l->GetTexture();
		if (lT == NULL)
		{
			std::cout << "Texture error!" << std::endl;
			continue;
		}

		l->SetPos(tPos);
		tPos.x += lT->w*_size/100;

		_letters.push_back(l);
		Game::AddObject(l);
	}
}

void Text::DestroyLetters()
{
	while(!_letters.empty())
	{
		Game::Remove( _letters.back());
		_letters.pop_back();
	}
}

void Text::SetText(std::string const& text)
{
	if(_text != text)
	{
		DestroyLetters();
		_text = text;
		CreateLetters();
	}
}

std::string const& Text::GetText() const
{
	return _text;
}

void Text::Update()
{
	static bool up = true;

	if(_brightness > 1 || _brightness < 0)
		up = !up;

	if(up)
	{
		_brightness += 0.01f;
	}
	else
	{
		_brightness -= 0.01f;
	}
}

void Text::SetColour(Colour const& c)
{
	Entity::SetColour(c);
}
