#ifndef VECTOR_H
#define VECTOR_H

#include <cmath>
#define PI 3.14159

struct Vector
{
	float x,y,z;

	Vector():x(0), y(0), z(0) {}

	Vector(float x, float y):x(x), y(y),z(0) {}

	Vector(float x, float y, float z):x(x), y(y), z(z) {}

	float Magnitude() const
	{
		return sqrt(pow(x,2)+pow(y,2)+pow(z,2));
	}

	//Versor (unit) vector (direction)
	Vector Versor() const
	{
		Vector temp = Vector();

		if (Magnitude() != 0)
		{
			float factor = 1/Magnitude();
			temp.x = x*factor;
			temp.y = y*factor;
			temp.z = z*factor;
		}
		
		return temp;
	}

	void Rotate2D(float a)
	{
		RotateAbout2D(a,Vector());
	}

	void RotateAbout2D(float a, Vector const& p)
	{
		*this -= p;

		double rad = a*PI/180.0f;
		double sn = sin(rad);
		double cs = cos(rad);

		float tX = x;
		float tY = y;

		x = tX * cs - tY * sn;
		y = tY * cs + tX * sn;

		*this += p;
	}


	//Operators-----
	//Scalar product
	float operator*(Vector const& vect) const
	{
		return (x*vect.x + y*vect.y + z*vect.z);
	}

	Vector operator*(float f) const
	{
		return Vector(x*f,y*f,z*f);
	}

	Vector operator/(float f) const
	{
		float factor = 1/f;
		return Vector(x*factor,y*factor,z*factor);
	}

	Vector operator+(float f) const
	{
		return Vector(x+f,y+f,z+f);
	}

	Vector operator+(Vector const& vect) const
	{
		return Vector(x+vect.x,
			      y+vect.y,
			      z+vect.z);
	}

	Vector operator-(float f) const
	{
		return Vector(x-f,y-f,z-f);
	}

	Vector operator-(Vector const& vect) const
	{
		return Vector(x-vect.x,
			      y-vect.y,
			      z-vect.z);
	}

	Vector &operator+=(Vector const& vect)
	{
		x += vect.x;
		y += vect.y;
		z += vect.z;

		return *this;
	}

	Vector &operator+=(float f)
	{
		x += f;
		y += f;
		z += f;

		return *this;
	}

	Vector &operator-=(Vector const& vect)
	{
		x -= vect.x;
		y -= vect.y;
		z -= vect.z;

		return *this;
	}

	Vector &operator-=(float f)
	{
		x -= f;
		y -= f;
		z -= f;

		return *this;
	}

	Vector &operator*=(float f)
	{
		x *= f;
		y *= f;
		z *= f;

		return *this;
	}
	
	Vector &operator/=(float f)
	{
		float factor = 1/f;
		x *= factor;
		y *= factor;
		z *= factor;

		return *this;
	}

	Vector &operator=(float f)
	{
		x = f;
		y = f;
		z = f;

		return *this;
	}

	Vector operator-()
	{
		return Vector(-x,-y,-z);
	}
};
#endif
