#include <iostream>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>

#ifdef _WIN32
#include <Windows.h>
#include <GL/gl.h>
#include "SDL/SDL_syswm.h"
#endif

#include "Window.h"

Window::Window():_startWidth(448),_startHeight(496)
{
	_w = _startWidth;
	_h = _startHeight;
	_fullscreen = false;
}

Window::~Window()
{
	std::cout << "Closing SDL_image" << std::endl;
	IMG_Quit();
	std::cout << "Closing SDL_ttf" << std::endl;
	TTF_Quit();
	std::cout << "Closing SDL" << std::endl;
	SDL_Quit();
}

bool Window::Init()
{
	if(SDL_Init(SDL_INIT_EVERYTHING) == -1)
	{
		std::cout << "Error: Failed to initialize SDL (";
		std::cout << SDL_GetError() << ")" << std::endl;
		return false;
	}

	if(TTF_Init() == -1)
	{
		std::cout << "Error: Failed to initialize TTF support (";
		std::cout << TTF_GetError() << ")" << std::endl;
		return false;
	}

	if(IMG_Init(IMG_INIT_PNG) != IMG_INIT_PNG)
	{
		std::cout << "Error: Failed to load SDL_image (";
		std::cout << IMG_GetError() << ")" << std::endl;
		return false;
	}

	SDL_VideoInfo const* info = SDL_GetVideoInfo();

	_wFS = info->current_w;
	_hFS = info->current_h;

	_flags = SDL_OPENGL;
	_flags |= SDL_GL_DOUBLEBUFFER;
	_flags |= SDL_HWPALETTE;
	_flags |= SDL_RESIZABLE;

	if(info->hw_available)
	{
		_flags |= SDL_HWSURFACE;
	}
	else
	{
		_flags |= SDL_SWSURFACE;
	}

	if(info->blit_hw)
	{
		_flags |= SDL_HWACCEL;
	}

	if(info->wm_available)
	{
		SDL_WM_SetCaption("Pachdad",NULL);
		//SDL_WM_SetIcon();
	}

	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	if(!SDL_SetVideoMode(_w,_h,32,_flags))
	{
		std::cout << "Error: Could not set video mode (";
		std::cout << SDL_GetError() << ")" << std::endl;

		return false;
	} 

	return true;
}

void Window::ToggleFullscreen()
{
#ifdef _WIN32
	//OpenGL context storage courtesy of
    //http://www.bytehazard.com/code/sdlres.html

	SDL_SysWMinfo wmInfo;

	SDL_VERSION(&wmInfo.version);
	if (SDL_GetWMInfo(&wmInfo) == -1)
	{
		std::cout << "Could not retrieve window manager information (";
		std::cout << SDL_GetError() << std::endl;
		return;
	}

	//Copy current device context
	HDC tempDC = GetDC(wmInfo.window);

	//Create new rendering context
	HGLRC tempRC = wglCreateContext(tempDC);
	if(tempRC == NULL)
	{
		std::cout << "Could not create temporary rendering context" << std::endl;
		return;
	}

	//Copy existing rendering context data
	if (!wglShareLists(wmInfo.hglrc, tempRC)) 
	{
		std::cout << "Could not copy rendering context data" << std::endl;
		return;
	}
#endif

	SDL_Surface* screen = NULL;
	if(_fullscreen)
	{
		screen = SDL_SetVideoMode(_startWidth,_startHeight,0,_flags);
		SDL_ShowCursor(1);
		_fullscreen = false;
		_w = screen->w;
		_h = screen->h;
		std::cout << "Changing window size: ";
		std::cout << _w << ", " << _h << std::endl;
	}
	else
	{
		screen = SDL_GetVideoSurface();
		screen = SDL_SetVideoMode(_wFS,_hFS,0,_flags ^ SDL_FULLSCREEN);
		if(screen == NULL) screen = SDL_SetVideoMode(0,0,0,_flags);
		else
		{
			SDL_ShowCursor(0);
			_fullscreen = true;
			_w = screen->w;
			_h = screen->h;
			std::cout << "Changing window size: ";
			std::cout << _w << ", " << _h << std::endl;
		}
	}

#ifdef _WIN32
	SDL_VERSION(&wmInfo.version);
	if (SDL_GetWMInfo(&wmInfo) == -1)
	{
		std::cout << "Could not retrieve window manager information (";
		std::cout << SDL_GetError() << ")" << std::endl;
	}

	if (!wglShareLists(tempRC, wmInfo.hglrc))
	{
		std::cout << "Could not restore rendering context data" << std::endl;
	}
	
	//Delete temporary rendering context
	if (!wglDeleteContext(tempRC))
	{
		std::cout << "Could not delete temporary rendering context" << std::endl;
	}
#endif
}

bool Window::IsFullscreen() const { return _fullscreen; }
int Window::GetWidth() const 
{ 
	if(!_fullscreen) return _w;
	else return _wFS;
}
int Window::GetHeight() const
{
	if(!_fullscreen) return _h;
	else return _hFS;
}
